@@ -5,11 +5,12 @@ from astroid import nodes
 from pylint.checkers import BaseChecker
 from pylint.checkers.utils import only_required_for_messages
 
+from python_ta.utils import _is_in_main
+
 FORBIDDEN_BUILTIN = ["input", "print", "open"]
 
 
 class IOFunctionChecker(BaseChecker):
-
     name = "IO_Function"
     msgs = {
         "E9998": (
@@ -52,7 +53,6 @@ class IOFunctionChecker(BaseChecker):
             if not (name in node.frame() or name in node.root()):
                 scope = node.scope()
                 scope_parent = scope.parent
-                # TODO: Only FunctionDefs are checked. Include global scope?
 
                 if (
                     isinstance(scope_parent, nodes.ClassDef)
@@ -68,6 +68,9 @@ class IOFunctionChecker(BaseChecker):
                 ):
                     if name in self.linter.config.forbidden_io_functions:
                         self.add_message("forbidden-IO-function", node=node, args=name)
+                elif isinstance(scope, nodes.Module) and not _is_in_main(node):
+                    if name in self.linter.config.forbidden_io_functions:
+                        self.add_message("forbidden-IO-function", node=node, args=name)
 
 
 def register(linter):
