@@ -16,7 +16,7 @@ from types import CodeType, FunctionType, ModuleType
 from typing import Any, Callable, List, Optional, Set, Tuple
 
 import wrapt
-from typeguard import check_type
+from typeguard import CollectionCheckStrategy, TypeCheckError, check_type
 
 # Configuration options
 
@@ -137,57 +137,54 @@ def add_class_invariants(klass: type) -> None:
         # This means the class has already been decorated
         return
 
-    # Update representation invariants from this class' docstring and those of its superclasses.
-    rep_invariants: List[Tuple[str, CodeType]] = []
+    _set_invariants(klass)
 
-    # Iterate over all inherited classes except builtins
-    for cls in reversed(klass.__mro__):
-        if "__representation_invariants__" in cls.__dict__:
-            rep_invariants.extend(cls.__representation_invariants__)
-        elif cls.__module__ != "builtins":
-            assertions = parse_assertions(cls, parse_token="Representation Invariant")
-            # Try compiling assertions
-            for assertion in assertions:
-                try:
-                    compiled = compile(assertion, "<string>", "eval")
-                except:
-                    _debug(
-                        f"Warning: representation invariant {assertion} could not be parsed as a valid Python expression"
-                    )
-                    continue
-                rep_invariants.append((assertion, compiled))
-
-    setattr(klass, "__representation_invariants__", rep_invariants)
+    klass_mod = _get_module(klass)
+    cls_annotations = None  # This is a cached value set the first time new_setattr is called
 
     def new_setattr(self: klass, name: str, value: Any) -> None:
         """Set the value of the given attribute on self to the given value.
 
         Check representation invariants for this class when not within an instance method of the class.
         """
-        klass_mod = _get_module(klass)
-        cls_annotations = typing.get_type_hints(klass, globalns=klass_mod.__dict__)
+        if not ENABLE_CONTRACT_CHECKING:
+            super(klass, self).__setattr__(name, value)
+            return
+
+        nonlocal cls_annotations
+        if cls_annotations is None:
+            cls_annotations = typing.get_type_hints(klass, localns=klass_mod.__dict__)
 
         if name in cls_annotations:
             try:
                 _debug(f"Checking type of attribute {attr} for {klass.__qualname__} instance")
-                check_type(name, value, cls_annotations[name])
-            except TypeError:
+                check_type(
+                    value,
+                    cls_annotations[name],
+                    collection_check_strategy=CollectionCheckStrategy.ALL_ITEMS,
+                )
+            except TypeCheckError:
                 raise AssertionError(
                     f"Value {_display_value(value)} did not match type annotation for attribute "
                     f"{name}: {_display_annotation(cls_annotations[name])}"
                 ) from None
-
+        original_attr_value_exists = False
+        original_attr_value = None
+        if hasattr(super(klass, self), name):
+            original_attr_value_exists = True
+            original_attr_value = super(klass, self).__getattribute__(name)
         super(klass, self).__setattr__(name, value)
-        curframe = inspect.currentframe()
-        callframe = inspect.getouterframes(curframe, 2)
-        frame_locals = callframe[1].frame.f_locals
+        frame_locals = inspect.currentframe().f_back.f_locals
         if self is not frame_locals.get("self"):
             # Only validating if the attribute is not being set in a instance/class method
-            klass_mod = _get_module(klass)
-            if klass_mod is not None and ENABLE_CONTRACT_CHECKING:
+            if klass_mod is not None:
                 try:
                     _check_invariants(self, klass, klass_mod.__dict__)
                 except PyTAContractError as e:
+                    if original_attr_value_exists:
+                        super(klass, self).__setattr__(name, original_attr_value)
+                    else:
+                        super(klass, self).__delattr__(name)
                     raise AssertionError(str(e)) from None
 
     for attr, value in klass.__dict__.items():
@@ -216,7 +213,7 @@ def _check_function_contracts(wrapped, instance, args, kwargs):
             try:
                 _debug(f"Checking type of parameter {param} in call to {wrapped.__qualname__}")
                 check_type_strict(param, arg, annotations[param])
-            except TypeError:
+            except (TypeError, TypeCheckError):
                 additional_suggestions = _get_argument_suggestions(arg, annotations[param])
 
                 raise PyTAContractError(
@@ -257,7 +254,7 @@ def _check_function_contracts(wrapped, instance, args, kwargs):
         try:
             _debug(f"Checking return type from call to {wrapped.__qualname__}")
             check_type_strict("return", r, return_type)
-        except TypeError:
+        except (TypeError, TypeCheckError):
             raise PyTAContractError(
                 f"{wrapped.__name__}'s return value {_display_value(r)} did not match "
                 f"return type annotation {_display_annotation(return_type)}"
@@ -304,7 +301,9 @@ def check_type_strict(argname: str, value: Any, expected_type: type) -> None:
             type(value) is bool and expected_type is int
         ):
             raise TypeError(f"type of {argname} must be {expected_type}; got {value} instead")
-        check_type(argname, value, expected_type)
+        check_type(
+            value, expected_type, collection_check_strategy=CollectionCheckStrategy.ALL_ITEMS
+        )
 
 
 def _get_argument_suggestions(arg: Any, annotation: type) -> str:
@@ -365,14 +364,16 @@ def _check_class_type_annotations(klass: type, instance: Any) -> None:
         - isinstance(instance, klass)
     """
     klass_mod = _get_module(klass)
-    cls_annotations = typing.get_type_hints(klass, globalns=klass_mod.__dict__)
+    cls_annotations = typing.get_type_hints(klass, localns=klass_mod.__dict__)
 
     for attr, annotation in cls_annotations.items():
         value = getattr(instance, attr)
         try:
             _debug(f"Checking type of attribute {attr} for {klass.__qualname__} instance")
-            check_type(attr, value, annotation)
-        except TypeError:
+            check_type(
+                value, annotation, collection_check_strategy=CollectionCheckStrategy.ALL_ITEMS
+            )
+        except TypeCheckError:
             raise AssertionError(
                 f"{_display_value(value)} did not match type annotation for attribute {attr}: "
                 f"{_display_annotation(annotation)}"
@@ -598,3 +599,39 @@ def _debug(msg: str) -> None:
         return
 
     print("[PyTA]", msg, file=sys.stderr)
+
+
+def _set_invariants(klass: type) -> None:
+    """Retrieve and set the representation invariants of this class"""
+    # Update representation invariants from this class' docstring and those of its superclasses.
+    rep_invariants: List[Tuple[str, CodeType]] = []
+
+    # Iterate over all inherited classes except builtins
+    for cls in reversed(klass.__mro__):
+        if "__representation_invariants__" in cls.__dict__:
+            rep_invariants.extend(cls.__representation_invariants__)
+        elif cls.__module__ != "builtins":
+            assertions = parse_assertions(cls, parse_token="Representation Invariant")
+            # Try compiling assertions
+            for assertion in assertions:
+                try:
+                    compiled = compile(assertion, "<string>", "eval")
+                except:
+                    _debug(
+                        f"Warning: representation invariant {assertion} could not be parsed as a valid Python expression"
+                    )
+                    continue
+                rep_invariants.append((assertion, compiled))
+
+    setattr(klass, "__representation_invariants__", rep_invariants)
+
+
+def validate_invariants(obj: object) -> None:
+    """Check that the representation invariants of obj are satisfied."""
+    klass = obj.__class__
+    klass_mod = _get_module(klass)
+
+    try:
+        _check_invariants(obj, klass, klass_mod.__dict__)
+    except PyTAContractError as e:
+        raise AssertionError(str(e)) from None
